"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPath = getPath;
const pathfinding_1 = require("pathfinding");
const mapCells_js_1 = require("./mapper/mapCells.js");
const mapPathfindingDiagonalMovement_js_1 = require("./mapper/mapPathfindingDiagonalMovement.js");
const mapWalkableMatrix_js_1 = require("./mapper/mapWalkableMatrix.js");
const mapFinder_js_1 = require("./mapper/mapFinder.js");
const mapHeuristic_js_1 = require("./mapper/mapHeuristic.js");
/**
 * @param grid The grid
 * @param start The coordinate that the path should start
 * @param end The coordinate that the path should end
 * @param options The options to customize the pathfinding
 * @returns The shortest path including the start and the end cell
 */
function getPath(grid, start, end, options) {
    const { algorithm = 'A_STAR', diagonalMovement = 'ALWAYS', heuristic = 'MANHATTAN', isWalkable = () => true, } = options || {};
    const matrix = (0, mapWalkableMatrix_js_1.mapWalkableMatrix)(grid, isWalkable);
    const pathfindingGrid = new pathfinding_1.Grid(matrix);
    const finder = (0, mapFinder_js_1.mapFinder)(algorithm, {
        diagonalMovement: (0, mapPathfindingDiagonalMovement_js_1.mapPathfindingDiagonalMovement)(diagonalMovement),
        heuristic: (0, mapHeuristic_js_1.mapHeuristic)(grid, heuristic),
    });
    const path = finder.findPath(start.col, start.row, end.col, end.row, pathfindingGrid);
    return (0, mapCells_js_1.mapCells)(grid, path);
}
